import numpy as np


def read_bin(path: str, shape=None, dtype='float32'):
    if not shape:
        shape = get_shape(path)
    dat = np.fromfile(path, dtype=dtype)
    dat = np.reshape(dat, shape)
    return dat


def write_bin(data: np.ndarray, fn: str, add_shape=True, dtype='float32'):
    data = np.array(data, dtype=dtype)
    if fn.endswith('.bin'):
        fn = fn[:-4]
    if add_shape:
        data.tofile(f'{fn}_{"x".join(map(str, data.shape))}.bin')
    else:
        data.tofile(f'{fn}.bin')


def get_shape(fname: str):
    # 'xxx_5x3.bin' --> (5, 3)
    fname_len = len(fname)
    fname_inverse = fname[::-1]
    width_right = fname_len - 4
    depth_left = fname_len - fname_inverse.find('_')
    shape = tuple(map(int, fname[depth_left:width_right].split('x')))
    return shape



from PIL import Image
import io
import numpy as np
import os
try:
    import fitz
except ModuleNotFoundError:
    print('WARNING: Module "fitz" not found, cannot use "pic2pdf".\n'
          'For further information, visit https://pypi.org/project/PyMuPDF/.')


def cut_img(img, loc=[0, 0], show_info=False):
    w, h = img.size
    img = np.array(img)
    cut_color = img[loc[0], loc[1]]
    top, bottom, left, right = 0, h, 0, w
    for ih in range(h):
        if not (cut_color == img[ih]).all():
            top = ih
            break
    for ih in range(h):
        if not (cut_color == img[h - ih - 1]).all():
            bottom = h - ih
            break
    for iw in range(w):
        if not (cut_color == img[top:bottom, iw]).all():
            left = iw
            break
    for iw in range(w):
        if not (cut_color == img[top:bottom, w - iw - 1]).all():
            right = w - iw
            break
    if show_info:
        print(f'WxH: {w}x{h} -> {right-left}x{bottom-top}')
    return Image.fromarray(img[top:bottom, left:right])


def savefig(fig, fname, **kwargs):
    dpi = kwargs['dpi'] if 'dpi' in kwargs.keys() else 96
    buffer = io.BytesIO()
    fig.savefig(buffer, format=fname.split('.')[-1], **kwargs)
    buffer.seek(0)
    image = Image.open(buffer)
    if 'tif' in fname.split('.')[-1]:
        cut_img(image).save(fname, dpi=(dpi, dpi), compression='tiff_lzw')
    else:
        cut_img(image).save(fname, dpi=(dpi, dpi))


def figure_cutter(figure_ID_old, figure_ID_new=None, **kwargs):
    if not figure_ID_new:
        figure_ID_new = figure_ID_old
    img = Image.open(figure_ID_old)
    try:
        dpi = img.info['dpi'][0]
    except:
        print('Can’t get dpi, set dpi=96')
        dpi = 96
    if 'tif' in figure_ID_new.split('.')[-1]:
        cut_img(img, **kwargs).save(figure_ID_new, dpi=(dpi, dpi), compression='tiff_lzw')
    else:
        cut_img(img, **kwargs).save(figure_ID_new, dpi=(dpi, dpi))


def pic2pdf(img_path, pdf_path=None, pdf_name=None):
    img_name = os.path.basename(img_path).split('.')[0]
    if not pdf_name:
        pdf_name = img_name
    if not pdf_path:
        pdf_path = os.path.join(os.path.dirname(img_path), pdf_name)
    doc = fitz.open()
    imgdoc = fitz.open(img_path)
    # 使用图片创建单页的PDF
    pdfbytes = imgdoc.convertToPDF()
    imgpdf = fitz.open('pdf', pdfbytes)
    # 将当前页写入文档
    doc.insertPDF(imgpdf)
    doc.save(pdf_path)
    doc.close()


def changedpi(infile, outfile=None, dpi=300):
    if not outfile:
        outfile = infile
    im = Image.open(infile)
    if 'tif' in outfile.split('.')[-1]:
        im.save(outfile, dpi=(dpi, dpi), compression='tiff_lzw')
    else:
        im.save(outfile, dpi=(dpi, dpi))


def list_processing(func, fn_old_dir, fn_new_dir=None, format='tif', **kwargs):
    if not fn_new_dir:
        fn_new_dir = fn_old_dir
    else:
        os.makedirs(fn_new_dir, exist_ok=True)
    for fn in os.listdir(fn_old_dir):
        func(os.path.join(fn_old_dir, fn), os.path.join(fn_new_dir, f'{fn.split(".")[0]}.{format}'), **kwargs)


if __name__ == '__main__':
    figure_cutter('t1.tif', 't2.tif')
    list_processing(pic2pdf, 'img', format='pdf')



import setuptools

with open("README.md", "r") as f:
    long_description = f.read()

setuptools.setup(
    name="pyseismic",
    version="0.0.6",
    author="Xintao Chai",
    author_email="xtchai@126.com",
    description="A package for seismic data processing",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/XintaoChai",
    packages=setuptools.find_packages(),
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
)



from matplotlib import pyplot as plt
from matplotlib.colors import ListedColormap
import numpy as np


def promax():
    taper = np.linspace(0, 1, 128)
    taper2 = np.concatenate((np.zeros(64), np.linspace(0, 1, 64)))
    r = np.concatenate((taper, np.ones(128)))
    g = np.concatenate((taper, taper[::-1]))
    b = np.concatenate((taper, taper2[::-1]))
    r = np.expand_dims(r, axis=1)
    g = np.expand_dims(g, axis=1)
    b = np.expand_dims(b, axis=1)
    rgb = np.concatenate((r, g, b), axis=1)
    colormap = ListedColormap(rgb)
    return colormap


if __name__ == '__main__':
    x = np.random.random([16, 16])
    plt.figure()
    plt.imshow(x, cmap=promax())
    plt.colorbar()
    plt.show()
